"use strict";
/**
 * Timer
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Timer
 */
var Timer = /** @class */ (function () {
    /**
     * 생성자
     * @param interval interval 시간간격 안에 등록된 callback들이 차례대로 실행된다.
     * @param minTick callback과 callback사이에 최소 시간간격을 설정할 수 있다.
     */
    function Timer(interval, minTick) {
        this.interval = interval;
        this.minTick = minTick;
        this._iterateList = [];
        this._onceList = [];
        this._id = 0;
        this._t = null;
        this._alter = true;
        if (!minTick) {
            this.minTick = 0;
        }
    }
    Timer.prototype.iterate = function (callback) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        this._iterateList.push({
            id: ++this._id,
            callback: callback,
            args: args,
        });
        this.start();
        return this._id;
    };
    Timer.prototype.remove = function (id) {
        var before = this._iterateList;
        this._iterateList = this._iterateList.filter(function (o) {
            return o.id !== id;
        });
        return before.length !== this._iterateList.length;
    };
    Timer.prototype.once = function (target) {
        var _this = this;
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return new Promise(function (resolve, reject) {
            _this._onceList.push({
                id: 0,
                callback: function () {
                    var args = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        args[_i] = arguments[_i];
                    }
                    target.apply(void 0, args).then(function (value) {
                        resolve(value);
                    }).catch(function (err) {
                        reject(err);
                    });
                },
                args: args,
            });
            _this.start();
        });
    };
    Timer.prototype.start = function () {
        this.stop();
        var count = this._iterateList.length + this._onceList.length;
        if (count === 0) {
            return;
        }
        var tick = this.interval / count;
        if (this.minTick) {
            tick = Math.max(tick, this.minTick);
        }
        this._t = setInterval(this._do.bind(this), tick);
    };
    Timer.prototype.stop = function () {
        if (this._t) {
            clearInterval(this._t);
            this._t = null;
        }
    };
    Object.defineProperty(Timer.prototype, "isStarted", {
        get: function () {
            return !!this._t;
        },
        enumerable: true,
        configurable: true
    });
    Timer.prototype._nextCallback = function () {
        var pack;
        if (this._alter) {
            pack = this._iterateList.shift();
            if (pack) {
                this._iterateList.push(pack);
                this._alter = !this._alter;
            }
            else {
                pack = this._onceList.shift();
            }
        }
        else {
            pack = this._onceList.shift();
            if (pack) {
                this._alter = !this._alter;
            }
            else {
                pack = this._iterateList.shift();
                if (pack) {
                    this._iterateList.push(pack);
                }
            }
        }
        return pack;
    };
    Timer.prototype._do = function () {
        var pack = this._nextCallback();
        if (pack) {
            pack.callback.apply(pack, pack.args);
        }
        else {
            this.stop();
        }
    };
    return Timer;
}());
exports.Timer = Timer;
exports.default = Timer;

//# sourceMappingURL=index.js.map
