import * as fecha from 'fecha'


export interface IWriter {
  log: (msg: string) => void
  error: (msg: string) => void
  link?: IWriter
}


class DefaultWriter implements IWriter {
  private _other: IWriter

  log(msg: string): void {
    console.log(msg)
    if(this._other) {
      this._other.log(msg)
    }
  }

  error(msg: string): void {
    console.log(msg)
    if(this._other) {
      this._other.error(msg)
    }
  }

  set link(other: IWriter) {
    this._other = other
  }
}


fecha.masks.myMask = 'YYYY-MM-DD HH:mm:ss.SSS'
let _writer: IWriter = new DefaultWriter()

export class Logger {
  static get writer(): IWriter {
    return _writer
  }

  static set writer(writer: IWriter)  {
    _writer = writer
  }

  constructor(public readonly name?: string) {
  }

  protected _makeMsg(msgs: any[]) {
    let time = fecha.format(new Date(), 'myMask')
    if(this.name) {
      return `${time} > ${this.name} - ${msgs.join(' ')}` 
    } else {
      return `${time} > ${msgs.join(' ')}` 
    }
  }

  log(...msgs: any[]): void {
    let msg = this._makeMsg(msgs)
    if(Logger.writer) {
      Logger.writer.log(msg)
    }
  }

  error(...msgs: any[]): void {
    let msg = this._makeMsg(msgs)
    if(Logger.writer) {
      Logger.writer.error(msg)
    }
  }
}

export default Logger