/**
 * test promisify
 */


import * as $4 from '../fourdollar'
import {expect} from 'chai'

describe('promisify()', () => {
  const testObj = {
    name: 'bynaki',
    greet(err: Error, say: string, callback: Function): void {
      callback.apply(this, [err, `${say}, ${this.name}`])
    },
    _greet: null,
  }

  it('testObj', done => {
    testObj.greet(new Error('its error!!'), 'hello'
      , (err: Error, greeting: string): void => {
      expect(err.message).to.be.equal('its error!!')
      expect(greeting).to.be.equal('hello, bynaki')
      done()
    })
  })

  it('기본인수 전달', () => {
    testObj._greet = $4.promisify(testObj.greet)
    return testObj._greet(undefined, 'hello')
    .then(greeting => {
      expect(greeting).to.be.equal('hello, bynaki')
    })
  })

  it('에러 처리', () => {
    return testObj._greet(new Error('its error!!'), 'hello')
    .catch((err: Error) => {
      expect(err.message).to.be.equal('its error!!')
    })
  })

  it('context를 지정하지 않았을 때 greet 메서드가 this를 참조하므로 에러가 나야한다.'
    , () => {
    const greet = $4.promisify(testObj.greet)
    return greet(undefined, 'hello')
    .catch((err: Error) => {
      expect(err).to.be.an('error')
    })
  })

  it('에러를 반환하지 않는 콜백이라면', () => {
    testObj._greet = $4.promisify(testObj.greet, false)
    return testObj._greet('not error!!', 'hello')
    .then(args => {
      expect(args).to.be.length(2)
      expect(args[0]).to.be.equal('not error!!')
      expect(args[1]).to.be.equal('hello, bynaki')
    })
  })

  it('context를 직접 지정한다면', () => {
    const obj = {
      name: 'rabbit'
    }
    const greet = $4.promisify(testObj.greet, obj)
    return greet(undefined, 'hello')
    .then(greeting => {
      expect(greeting).to.be.equal('hello, rabbit')
    })
  })

  it('hasErr와 context 둘다 지정', () => {
    const greet = $4.promisify<(err: Error, say: string) => Promise<any[]>>(
      testObj.greet, false, testObj)
    return greet('not error!!', 'hello')
    .then(args => {
      expect(args).to.be.length(2)
      expect(args[0]).to.be.equal('not error!!')
      expect(args[1]).to.be.equal('hello, bynaki')
    })
  })

  it('hasErr와 context를 순서를 바꿔 지정', () => {
    const greet = $4.promisify(testObj.greet, testObj, true)
    return greet(undefined, 'hello')
    .then(greeting => {
      expect(greeting).to.be.equal('hello, bynaki')
    })
  })

  it('엄한 인수 전달', done => {
    try {
      const greet = $4.promisify(testObj.greet, testObj, 33)
    } catch(err) {
      done()
    }
  })

  it('async-await', () => {
    const greet = $4.promisify(testObj.greet, testObj)
    return (async () => {
      const greeting: string = await greet(undefined, 'hello')
      expect(greeting).to.be.equal('hello, bynaki')
    })()
  })
})